@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2010 Derick Rethans                               |
+   | Copyright (c) 2002-2012 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -40,6 +40,10 @@
 #include "ext/standard/flock_compat.h"
 #include "main/php_ini.h"
 
+#ifndef NAME_MAX
+# define NAME_MAX (MAXNAMELEN-1)
+#endif
+
 #define READ_BUFFER_SIZE 128
 
 char* xdebug_fd_read_line_delim(int socket, fd_buf *context, int type, unsigned char delim, int *length)
@@ -425,6 +429,7 @@ FILE *xdebug_fopen(char *fname, char *mode, char *extension, char **new_fname)
 	FILE *fh;
 	struct stat buf;
 	char *tmp_fname = NULL;
+	int   filename_len = 0;
 
 	/* We're not doing any tricks for append mode... as that has atomic writes
 	 * anyway. And we ignore read mode as well. */
@@ -432,6 +437,14 @@ FILE *xdebug_fopen(char *fname, char *mode, char *extension, char **new_fname)
 		return xdebug_open_file(fname, mode, extension, new_fname);
 	}
 
+	/* Make sure we don't open a file with a path that's too long */
+	filename_len += (fname ? strlen(fname) : 0); /* filename */
+	filename_len += (extension ? strlen(extension) : 0) + 1; /* extension (+ ".") */
+	filename_len += 8; /* possible random extension (+ ".") */
+	if (filename_len > NAME_MAX) {
+		fname[NAME_MAX - (extension ? strlen(extension) : 0 )] = '\0';
+	}
+
 	/* In write mode however we do have to do some stuff. */
 	/* 1. Check if the file exists */
 	if (extension) {
@@ -589,9 +602,10 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 						if (retval == SUCCESS) {
 							strval = estrdup(Z_STRVAL_PP(data));
 							
-							/* replace slashes, dots, question marks, plus signs,
-							 * ambersands and spaces with underscores */
-							while ((char_ptr = strpbrk(strval, "/\\.?&+ ")) != NULL) {
+							/* replace slashes, dots, question marks, plus
+							 * signs, ampersands, spaces and other evil chars
+							 * with underscores */
+							while ((char_ptr = strpbrk(strval, "/\\.?&+:*\"<>| ")) != NULL) {
 								char_ptr[0] = '_';
 							}
 							xdebug_str_add(&fname, strval, 0);
