@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2010 Derick Rethans                               |
+   | Copyright (c) 2002-2012 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -70,9 +70,9 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC);
 void (*xdebug_old_execute_internal)(zend_execute_data *current_execute_data, int return_value_used TSRMLS_DC);
 void xdebug_execute_internal(zend_execute_data *current_execute_data, int return_value_used TSRMLS_DC);
 
-/* error callback repalcement functions */
-void (*old_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
-void (*new_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
+/* error callback replacement functions */
+void (*xdebug_old_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
+void (*xdebug_new_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
 void xdebug_error_cb(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
 
 static int xdebug_header_handler(sapi_header_struct *h XG_SAPI_HEADER_OP_DC, sapi_headers_struct *s TSRMLS_DC);
@@ -283,7 +283,7 @@ PHP_INI_BEGIN()
 	PHP_INI_ENTRY("xdebug.remote_mode",           "req",                PHP_INI_ALL,    OnUpdateDebugMode)
 	STD_PHP_INI_ENTRY("xdebug.remote_port",       "9000",               PHP_INI_ALL,    OnUpdateLong,   remote_port,       zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_BOOLEAN("xdebug.remote_autostart","0",                  PHP_INI_ALL,    OnUpdateBool,   remote_autostart,  zend_xdebug_globals, xdebug_globals)
-	STD_PHP_INI_BOOLEAN("xdebug.remote_connect_back","0",               PHP_INI_SYSTEM, OnUpdateBool,   remote_connect_back,  zend_xdebug_globals, xdebug_globals)
+	STD_PHP_INI_BOOLEAN("xdebug.remote_connect_back","0",               PHP_INI_ALL,    OnUpdateBool,   remote_connect_back,  zend_xdebug_globals, xdebug_globals)
 	STD_PHP_INI_ENTRY("xdebug.remote_log",        "",                   PHP_INI_ALL,    OnUpdateString, remote_log,        zend_xdebug_globals, xdebug_globals)
 	PHP_INI_ENTRY("xdebug.idekey",                "",                   PHP_INI_ALL,    OnUpdateIDEKey)
 	STD_PHP_INI_ENTRY("xdebug.remote_cookie_expire_time", "3600",       PHP_INI_ALL,    OnUpdateLong,   remote_cookie_expire_time, zend_xdebug_globals, xdebug_globals)
@@ -469,7 +469,7 @@ static int xdebug_include_or_eval_handler(ZEND_OPCODE_HANDLER_ARGS)
 		if (XG(last_eval_statement)) {
 			efree(XG(last_eval_statement));
 		}
-		XG(last_eval_statement) = php_addcslashes(Z_STRVAL_P(inc_filename), Z_STRLEN_P(inc_filename), &tmp_len, 0, "'\\\0..\37", 6 TSRMLS_CC);
+		XG(last_eval_statement) = estrndup(Z_STRVAL_P(inc_filename), Z_STRLEN_P(inc_filename));
 
 		if (inc_filename == &tmp_inc_filename) {
 			zval_dtor(&tmp_inc_filename);
@@ -500,23 +500,31 @@ PHP_MINIT_FUNCTION(xdebug)
 	zend_execute_internal = xdebug_execute_internal;
 
 	/* Replace error handler callback with our own */
-	old_error_cb = zend_error_cb;
-	new_error_cb = xdebug_error_cb;
+	xdebug_old_error_cb = zend_error_cb;
+	xdebug_new_error_cb = xdebug_error_cb;
 
 	/* Get reserved offset */
 	zend_xdebug_global_offset = zend_get_resource_handle(&dummy_ext);
 
 	/* Overload the "exit" opcode */
-	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXIT);
+	XDEBUG_SET_OPCODE_OVERRIDE_ASSIGN(exit, ZEND_EXIT);
+
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMP);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPZ);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPNZ);
+	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_JMPZ_EX);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_IDENTICAL);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_NOT_IDENTICAL);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_EQUAL);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_NOT_EQUAL);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_SMALLER);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_IS_SMALLER_OR_EQUAL);
+	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_BOOL_NOT);
+
+	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD);
+	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_SUB);
+	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_MUL);
+	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_DIV);
 
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_ARRAY_ELEMENT);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_RETURN);
@@ -529,6 +537,7 @@ PHP_MINIT_FUNCTION(xdebug)
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_EXT_FCALL_BEGIN);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_CATCH);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_BOOL);
+	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_CHAR);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_ADD_STRING);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_INIT_ARRAY);
 	XDEBUG_SET_OPCODE_OVERRIDE_COMMON(ZEND_FETCH_DIM_R);
@@ -604,7 +613,7 @@ PHP_MSHUTDOWN_FUNCTION(xdebug)
 	zend_compile_file = old_compile_file;
 	zend_execute = xdebug_old_execute;
 	zend_execute_internal = xdebug_old_execute_internal;
-	zend_error_cb = old_error_cb;
+	zend_error_cb = xdebug_old_error_cb;
 
 	zend_hash_destroy(&XG(aggr_calls));
 
@@ -765,7 +774,7 @@ PHP_RINIT_FUNCTION(xdebug)
 	/* Hack: We check for a soap header here, if that's existing, we don't use
 	 * Xdebug's error handler to keep soap fault from fucking up. */
 	if (XG(default_enable) && zend_hash_find(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER]), "HTTP_SOAPACTION", 16, (void**)&dummy) == FAILURE) {
-		zend_error_cb = new_error_cb;
+		zend_error_cb = xdebug_new_error_cb;
 		zend_throw_exception_hook = xdebug_throw_exception_hook;
 	}
 	XG(remote_enabled) = 0;
@@ -807,9 +816,6 @@ PHP_RINIT_FUNCTION(xdebug)
 	orig->internal_function.handler = zif_xdebug_set_time_limit;
 
 	XG(headers) = xdebug_llist_alloc(xdebug_llist_string_dtor);
-	if (strcmp(sapi_module.name, "cli") == 0) {
-		SG(request_info).no_headers = 1;
-	}
 
 	return SUCCESS;
 }
@@ -968,9 +974,9 @@ static void xdebug_throw_exception_hook(zval *exception TSRMLS_DC)
 	file =    zend_read_property(default_ce, exception, "file",    sizeof("file")-1,    0 TSRMLS_CC);
 	line =    zend_read_property(default_ce, exception, "line",    sizeof("line")-1,    0 TSRMLS_CC);
 
-	if (Z_TYPE_P(message) != IS_STRING || Z_TYPE_P(file) != IS_STRING || Z_TYPE_P(line) != IS_LONG) {
-		php_error(E_ERROR, "Your exception class uses incorrect types for common properties: 'message' and 'file' need to be a string and 'line' needs to be an integer.");
-	}
+	convert_to_string_ex(&message);
+	convert_to_string_ex(&file);
+	convert_to_long_ex(&line);
 
 #if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 2) || PHP_MAJOR_VERSION >= 6
 	previous_exception = zend_read_property(default_ce, exception, "previous", sizeof("previous")-1, 1 TSRMLS_CC);
@@ -1044,11 +1050,7 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 	}
 	/* class->function breakpoints */
 	else if (fse->function.type == XFUNC_MEMBER || fse->function.type == XFUNC_STATIC_MEMBER) {
-		if (fse->function.type == XFUNC_MEMBER) {
-			tmp_name = xdebug_sprintf("%s->%s", fse->function.class, fse->function.function);
-		} else if( fse->function.type == XFUNC_STATIC_MEMBER) {
-			tmp_name = xdebug_sprintf("%s::%s", fse->function.class, fse->function.function);
-		}
+		tmp_name = xdebug_sprintf("%s::%s", fse->function.class, fse->function.function);
 
 		if (xdebug_hash_find(XG(context).function_breakpoints, tmp_name, strlen(tmp_name), (void *) &extra_brk_info)) {
 			/* Yup, breakpoint found, call handler if the breakpoint is not
@@ -1075,6 +1077,12 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 	xdebug_llist_element *le;
 	int                   eval_id = 0;
 
+	/* If we're evaluating for the debugger's eval capability, just bail out */
+	if (op_array && op_array->filename && strcmp("xdebug://debug-eval", op_array->filename) == 0) {
+		xdebug_old_execute(op_array TSRMLS_CC);
+		return;
+	}
+
 	/* if we're in a ZEND_EXT_STMT, we ignore this function call as it's likely
 	   that it's just being called to check for breakpoints with conditions */
 	if (edata && edata->opline && edata->opline->opcode == ZEND_EXT_STMT) {
@@ -1294,17 +1302,28 @@ void xdebug_execute(zend_op_array *op_array TSRMLS_DC)
 		}
 	}
 
-	/* If we're in an eval, we need to destroy the created ID again. */
-	if (XG(remote_enabled) && XG(context).handler->unregister_eval_id && fse->function.type == XFUNC_EVAL) {
-		XG(context).handler->unregister_eval_id(&(XG(context)), fse, eval_id);
-	}
-
 	fse->symbol_table = NULL;
 	fse->execute_data = NULL;
 	xdebug_llist_remove(XG(stack), XDEBUG_LLIST_TAIL(XG(stack)), xdebug_stack_element_dtor);
 	XG(level)--;
 }
 
+static int check_soap_call(function_stack_entry *fse)
+{
+	zend_module_entry tmp_mod_entry;
+
+	if (fse->function.class && 
+		(
+			(strstr(fse->function.class, "SoapClient") != NULL) ||
+			(strstr(fse->function.class, "SoapClient") != NULL)
+		) &&
+		(zend_hash_find(&module_registry, "soap", 5, (void**) &tmp_mod_entry) == SUCCESS)
+	) {
+		return 1;
+	}
+	return 0;
+}
+
 void xdebug_execute_internal(zend_execute_data *current_execute_data, int return_value_used TSRMLS_DC)
 {
 	zend_execute_data    *edata = EG(current_execute_data);
@@ -1313,6 +1332,9 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, int return
 	int                   do_return = (XG(do_trace) && XG(trace_file));
 	int                   function_nr = 0;
 
+	int                   restore_error_handler_situation = 0;
+	void                (*tmp_error_cb)(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args);
+
 	XG(level)++;
 	if (XG(level) == XG(max_nesting_level)) {
 		php_error(E_ERROR, "Maximum function nesting level of '%ld' reached, aborting!", XG(max_nesting_level));
@@ -1329,7 +1351,14 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, int return
 			XG(remote_enabled) = 0;
 		}
 	}
-	
+
+	/* Check for SOAP */
+	if (check_soap_call(fse)) {
+		restore_error_handler_situation = 1;
+		tmp_error_cb = zend_error_cb;
+		zend_error_cb = xdebug_old_error_cb;
+	}
+
 	if (XG(profiler_enabled)) {
 		xdebug_profiler_function_internal_begin(fse TSRMLS_CC);
 	}
@@ -1343,6 +1372,11 @@ void xdebug_execute_internal(zend_execute_data *current_execute_data, int return
 		xdebug_profiler_function_internal_end(fse TSRMLS_CC);
 	}
 
+	/* Restore SOAP situation if needed */
+	if (restore_error_handler_situation) {
+		zend_error_cb = tmp_error_cb;
+	}
+
 	xdebug_trace_function_end(fse, function_nr TSRMLS_CC);
 
 	/* Store return value in the trace file */
@@ -1540,37 +1574,25 @@ PHP_FUNCTION(xdebug_debug_zval_stdout)
 
 PHP_FUNCTION(xdebug_enable)
 {
-	zend_error_cb = new_error_cb;
+	zend_error_cb = xdebug_new_error_cb;
 	zend_throw_exception_hook = xdebug_throw_exception_hook;
 }
 
 PHP_FUNCTION(xdebug_disable)
 {
-	zend_error_cb = old_error_cb;
+	zend_error_cb = xdebug_old_error_cb;
 	zend_throw_exception_hook = NULL;
 }
 
 PHP_FUNCTION(xdebug_is_enabled)
 {
-	RETURN_BOOL(zend_error_cb == new_error_cb);
+	RETURN_BOOL(zend_error_cb == xdebug_new_error_cb);
 }
 
 PHP_FUNCTION(xdebug_break)
 {
-	char *file;
-	int   lineno;
-
-	if (XG(remote_enabled)) {
-		file = zend_get_executed_filename(TSRMLS_C);
-		lineno = zend_get_executed_lineno(TSRMLS_C);
-
-		if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), file, lineno, XDEBUG_BREAK, NULL, NULL)) {
-			XG(remote_enabled) = 0;
-		}
-		RETURN_TRUE;
-	} else {
-		RETURN_FALSE;
-	}
+	XG(context).do_break = 1;
+	RETURN_TRUE;
 }
 
 PHP_FUNCTION(xdebug_start_error_collection)
@@ -1693,6 +1715,10 @@ ZEND_DLEXPORT void xdebug_statement_call(zend_op_array *op_array)
 	int                   level = 0;
 	TSRMLS_FETCH();
 
+	if (!EG(current_execute_data)) {
+		return;
+	}
+
 	lineno = EG(current_execute_data)->opline->lineno;
 
 	file = op_array->filename;
