@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2010 Derick Rethans                               |
+   | Copyright (c) 2002-2012 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -423,6 +423,24 @@ static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_le
 				retval_p = *retval_pp;
 				goto cleanup;
 			}
+			element_length = name_length;
+
+			/* Then we try to see whether the first char is * and use the part between * and * as class name for the private property */
+			if (name[0] == '*') {
+				char *secondStar;
+				
+				secondStar = strstr(name + 1, "*");
+				if (secondStar) {
+					free(element);
+					element_length = name_length - (secondStar + 1 - name);
+					element = prepare_search_key(secondStar + 1, &element_length, name + 1, secondStar - name - 1);
+					if (ht && zend_hash_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
+						retval_p = *retval_pp;
+						goto cleanup;
+					}
+				}
+			}
+
 			break;
 	}
 cleanup:
@@ -453,6 +471,11 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 						keyword = *p + 1;
 						break;
 					}
+					if (*p[0] == ':') { /* special tricks */
+						keyword = *p;
+						state = 7;
+						break;
+					}
 					keyword = *p;
 					state = 1;
 					/* break intentionally missing */
@@ -487,6 +510,22 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 						}
 						state = 2;
 						type = XF_ST_OBJ_PROPERTY;
+					} else if (*p[0] == ':') {
+						keyword_end = *p;
+						if (keyword) {
+							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length TSRMLS_CC);
+							if (current_classname) {
+								efree(current_classname);
+							}
+							current_classname = NULL;
+							if (retval) {
+								current_classname = fetch_classname_from_zval(retval, &cc_length TSRMLS_CC);
+								st = CE_STATIC_MEMBERS(Z_OBJCE_P(retval));
+							}
+							keyword = NULL;
+						}
+						state = 8;
+						type = XF_ST_OBJ_PROPERTY;
 					}
 					break;
 				case 2:
@@ -495,6 +534,12 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 						state = 1;
 					}
 					break;
+				case 8:
+					if (*p[0] != ':') {
+						keyword = *p;
+						state = 1;
+					}
+					break;
 				case 3:
 					/* Associative arrays */
 					if (*p[0] == '\'' || *p[0] == '"') {
@@ -548,6 +593,25 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 						keyword = NULL;
 					}
 					break;
+				case 7: /* special cases, started with a ":" */
+					if (*p[0] == ':') {
+						state = 1;
+						keyword_end = *p;
+
+						if (strncmp(keyword, "::", 2) == 0) { /* static class properties */
+							zend_class_entry *ce = zend_fetch_class(XG(active_fse)->function.class, strlen(XG(active_fse)->function.class), ZEND_FETCH_CLASS_SELF TSRMLS_CC);
+							st = CE_STATIC_MEMBERS(ce);
+
+							current_classname = estrdup(ce->name);
+							cc_length = strlen(ce->name);
+							keyword = *p + 1;
+
+							type = XF_ST_OBJ_PROPERTY;
+						} else {
+							keyword = NULL;
+						}
+					}
+					break;
 			}
 			(*p)++;
 		}
@@ -558,6 +622,9 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 			st = fetch_ht_from_zval(retval TSRMLS_CC);
 		}
 	}
+	if (current_classname) {
+		efree(current_classname);
+	}
 	return retval;
 }
 
@@ -567,7 +634,7 @@ static xdebug_xml_node* get_symbol(char* name, int name_length, xdebug_var_expor
 
 	retval = get_symbol_contents_zval(name, name_length TSRMLS_CC);
 	if (retval) {
-		return xdebug_get_zval_value_xml_node(name, retval, options);
+		return xdebug_get_zval_value_xml_node(name, retval, options TSRMLS_CC);
 	}
 
 	return NULL;
@@ -1168,8 +1235,13 @@ DBGP_FUNC(breakpoint_set)
 
 static int xdebug_do_eval(char *eval_string, zval *ret_zval TSRMLS_DC)
 {
-	int              old_error_reporting;
-	int              res;
+	int                old_error_reporting;
+	int                res = FAILURE;
+	zval             **original_return_value_ptr_ptr = EG(return_value_ptr_ptr);
+	zend_op          **original_opline_ptr = EG(opline_ptr);
+	zend_op_array     *original_active_op_array = EG(active_op_array);
+	zend_execute_data *original_execute_data = EG(current_execute_data);
+	int                original_no_extensions = EG(no_extensions);
 
 	/* Remember error reporting level */
 	old_error_reporting = EG(error_reporting);
@@ -1177,12 +1249,21 @@ static int xdebug_do_eval(char *eval_string, zval *ret_zval TSRMLS_DC)
 
 	/* Do evaluation */
 	XG(breakpoints_allowed) = 0;
-	res = zend_eval_string(eval_string, ret_zval, "xdebug eval" TSRMLS_CC);
+
+	zend_first_try {
+		res = zend_eval_string(eval_string, ret_zval, "xdebug://debug-eval" TSRMLS_CC);
+	} zend_end_try();
 
 	/* Clean up */
 	EG(error_reporting) = old_error_reporting;
 	XG(breakpoints_allowed) = 1;
 
+	EG(return_value_ptr_ptr) = original_return_value_ptr_ptr;
+	EG(opline_ptr) = original_opline_ptr;
+	EG(active_op_array) = original_active_op_array;
+	EG(current_execute_data) = original_execute_data;
+	EG(no_extensions) = original_no_extensions;
+
 	return res;
 }
 
@@ -1210,9 +1291,7 @@ DBGP_FUNC(eval)
 	/* base64 decode eval string */
 	eval_string = (char*) xdebug_base64_decode((unsigned char*) CMD_OPTION('-'), strlen(CMD_OPTION('-')), &new_length);
 
-	zend_try {
-		res = xdebug_do_eval(eval_string, &ret_zval TSRMLS_CC);
-	} zend_end_try();
+	res = xdebug_do_eval(eval_string, &ret_zval TSRMLS_CC);
 
 	efree(eval_string);
 
@@ -1220,7 +1299,7 @@ DBGP_FUNC(eval)
 	if (res == FAILURE) {
 		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_EVALUATING_CODE);
 	} else {
-		ret_xml = xdebug_get_zval_value_xml_node(NULL, &ret_zval, options);
+		ret_xml = xdebug_get_zval_value_xml_node(NULL, &ret_zval, options TSRMLS_CC);
 		xdebug_xml_add_child(*retval, ret_xml);
 		zval_dtor(&ret_zval);
 	}
@@ -1638,6 +1717,7 @@ DBGP_FUNC(property_get)
 			XG(active_symbol_table) = fse->symbol_table;
 			XG(active_op_array)     = fse->op_array;
 			XG(This)                = fse->This;
+			XG(active_fse)          = fse;
 		} else {
 			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
 		}
@@ -1711,6 +1791,7 @@ DBGP_FUNC(property_set)
 			XG(active_symbol_table) = fse->symbol_table;
 			XG(active_op_array)     = fse->op_array;
 			XG(This)                = fse->This;
+			XG(active_fse)          = fse;
 		} else {
 			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
 		}
@@ -1824,6 +1905,7 @@ DBGP_FUNC(property_value)
 		XG(active_symbol_table) = fse->symbol_table;
 		XG(active_op_array)     = fse->op_array;
 		XG(This)                = fse->This;
+		XG(active_fse)          = fse;
 	} else {
 		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
 	}
@@ -1857,17 +1939,7 @@ static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, vo
 	if (contents) {
 		xdebug_xml_add_child(node, contents);
 	} else {
-		contents = xdebug_xml_node_init("property");
-		if (name[0] != '$') {
-			full_name = xdebug_sprintf("$%s", name);
-		} else {
-			full_name = xdstrdup(name);
-		}
-		xdebug_xml_add_attribute_ex(contents, "name", xdstrdup(name), 0, 1);
-		xdebug_xml_add_attribute_ex(contents, "fullname", full_name, 0, 1);
-
-		xdebug_xml_add_attribute(contents, "type", "uninitialized");
-		xdebug_xml_add_child(node, contents);
+		xdebug_attach_uninitialized_var(node, name);
 	}
 }
 
@@ -1925,6 +1997,15 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 			xdebug_hash_destroy(tmp_hash);
 		}
 
+		/* Check for static variables and constants, but only if it's a static
+		 * method call as we attach constants and static properties to "this"
+		 * too normally. */
+		if (fse->function.type == XFUNC_STATIC_MEMBER) {
+			zend_class_entry *ce = zend_fetch_class(fse->function.class, strlen(fse->function.class), ZEND_FETCH_CLASS_SELF TSRMLS_CC);
+
+			xdebug_attach_static_vars(node, options, ce TSRMLS_CC);
+		}
+
 		XG(active_symbol_table) = NULL;
 		XG(active_execute_data) = NULL;
 		XG(active_op_array)     = NULL;
@@ -2463,15 +2544,17 @@ int xdebug_dbgp_deinit(xdebug_con *context)
 		XG(stdio).php_header_write = NULL;
 	}
 
-	options = (xdebug_var_export_options*) context->options;
-	xdfree(options->runtime);
-	xdfree(context->options);
-	xdebug_hash_destroy(context->function_breakpoints);
-	xdebug_hash_destroy(context->exception_breakpoints);
-	xdebug_hash_destroy(context->eval_id_lookup);
-	xdebug_llist_destroy(context->line_breakpoints, NULL);
-	xdebug_hash_destroy(context->breakpoint_list);
-	xdfree(context->buffer);
+	if (XG(remote_enabled)) {
+		options = (xdebug_var_export_options*) context->options;
+		xdfree(options->runtime);
+		xdfree(context->options);
+		xdebug_hash_destroy(context->function_breakpoints);
+		xdebug_hash_destroy(context->exception_breakpoints);
+		xdebug_hash_destroy(context->eval_id_lookup);
+		xdebug_llist_destroy(context->line_breakpoints, NULL);
+		xdebug_hash_destroy(context->breakpoint_list);
+		xdfree(context->buffer);
+	}
 
 	if (XG(remote_log_file)) {
 		char *timestr = xdebug_get_time();
@@ -2481,6 +2564,7 @@ int xdebug_dbgp_deinit(xdebug_con *context)
 		fclose(XG(remote_log_file));
 		XG(remote_log_file) = NULL;
 	}
+	XG(remote_enabled) = 0;
 	return 1;
 }
 
@@ -2627,7 +2711,7 @@ int xdebug_dbgp_register_eval_id(xdebug_con *context, function_stack_entry *fse)
 
 	ei = xdcalloc(sizeof(xdebug_eval_info), 1);
 	ei->id = context->eval_id_sequence;
-	ei->contents = xdstrndup(fse->include_filename + 1, strlen(fse->include_filename) - 2);
+	ei->contents = xdstrndup(fse->include_filename, strlen(fse->include_filename));
 	ei->refcount = 2;
 
 	key = create_eval_key_file(fse->filename, fse->lineno);
@@ -2638,15 +2722,3 @@ int xdebug_dbgp_register_eval_id(xdebug_con *context, function_stack_entry *fse)
 
 	return ei->id;
 }
-
-int xdebug_dbgp_unregister_eval_id(xdebug_con *context, function_stack_entry *fse, int eval_id)
-{
-	char *key;
-
-	key = create_eval_key_file(fse->filename, fse->lineno);
-	xdebug_hash_delete(context->eval_id_lookup, key, strlen(key));
-
-	key = create_eval_key_id(eval_id);
-	xdebug_hash_delete(context->eval_id_lookup, key, strlen(key));
-	return 1;
-}
