@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2010 Derick Rethans                               |
+   | Copyright (c) 2002-2012 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -24,6 +24,7 @@
 #include "xdebug_str.h"
 #include "xdebug_superglobals.h"
 #include "xdebug_var.h"
+#include "ext/standard/html.h"
 
 #include "main/php_ini.h"
 
@@ -226,16 +227,30 @@ static void xdebug_append_error_head(xdebug_str *str, int html TSRMLS_DC)
 void xdebug_append_error_description(xdebug_str *str, int html, const char *error_type_str, char *buffer, const char *error_filename, const int error_lineno TSRMLS_DC)
 {
 	char **formats = html ? html_formats : text_formats;
+	char *escaped;
+#if PHP_VERSION_ID >= 50400
+	size_t newlen;
+#else
+	int    newlen;
+#endif
+
+	if (html) {
+		escaped = php_escape_html_entities(buffer, strlen(buffer), &newlen, 0, 0, NULL TSRMLS_CC);
+	} else {
+		escaped = estrdup(buffer);
+	}
 
 	if (strlen(XG(file_link_format)) > 0 && html) {
 		char *file_link;
 
 		create_file_link(&file_link, error_filename, error_lineno TSRMLS_CC);
-		xdebug_str_add(str, xdebug_sprintf(formats[11], error_type_str, buffer, file_link, error_filename, error_lineno), 1);
+		xdebug_str_add(str, xdebug_sprintf(formats[11], error_type_str, escaped, file_link, error_filename, error_lineno), 1);
 		xdfree(file_link);
 	} else {
-		xdebug_str_add(str, xdebug_sprintf(formats[1], error_type_str, buffer, error_filename, error_lineno), 1);
+		xdebug_str_add(str, xdebug_sprintf(formats[1], error_type_str, escaped, error_filename, error_lineno), 1);
 	}
+
+	efree(escaped);
 }
 
 void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
@@ -440,6 +455,17 @@ void xdebug_do_jit(TSRMLS_D)
 	}
 }
 
+static void php_output_error(const char *error TSRMLS_DC)
+{
+#ifdef PHP_DISPLAY_ERRORS_STDERR
+	if (PG(display_errors) == PHP_DISPLAY_ERRORS_STDERR) {
+		fputs(error, stderr);
+		fflush(stderr);
+		return;
+	}
+#endif
+	php_printf("%s", error);
+}
 
 /* Error callback for formatting stack traces */
 void xdebug_error_cb(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args)
@@ -469,7 +495,6 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 	PG(last_error_message) = strdup(buffer);
 	PG(last_error_file) = strdup(error_filename);
 	PG(last_error_lineno) = error_lineno;
-
 #if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
 	error_handling  = EG(error_handling);
 	exception_class = EG(exception_class);
@@ -540,13 +565,13 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 				xdebug_append_printable_stack(&str, PG(html_errors) TSRMLS_CC);
 				xdebug_str_add(&str, XG(last_exception_trace), 0);
 				xdebug_append_error_footer(&str, PG(html_errors));
-				php_printf("%s", str.d);
+				php_output_error(str.d TSRMLS_CC);
 
 				xdfree(str.d);
 				free(tmp_buf);
 			} else {
 				printable_stack = get_printable_stack(PG(html_errors), error_type_str, buffer, error_filename, error_lineno TSRMLS_CC);
-				php_printf("%s", printable_stack);
+				php_output_error(printable_stack TSRMLS_CC);
 				xdfree(printable_stack);
 			}
 		}
@@ -572,6 +597,8 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 	}
 	xdfree(error_type_str);
 
+#if PHP_VERSION_ID < 50400
+
 	/* Bail out if we can't recover */
 	switch (type) {
 		case E_CORE_ERROR:
@@ -597,6 +624,46 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 			return;
 	}
 
+#else
+
+	/* Bail out if we can't recover */
+	switch (type) {
+		case E_CORE_ERROR:
+			if (!php_get_module_initialized()) {
+				/* bad error in module startup - no way we can live with this */
+				exit(-2);
+			}
+		case E_ERROR:
+		case E_RECOVERABLE_ERROR:
+		case E_PARSE:
+		case E_COMPILE_ERROR:
+		case E_USER_ERROR:
+			EG(exit_status) = 255;
+			if (php_get_module_initialized()) {
+				if (!PG(display_errors) &&
+				    !SG(headers_sent) &&
+					SG(sapi_headers).http_response_code == 200
+				) {
+					sapi_header_line ctr = {0};
+
+					ctr.line = "HTTP/1.0 500 Internal Server Error";
+					ctr.line_len = sizeof("HTTP/1.0 500 Internal Server Error") - 1;
+					sapi_header_op(SAPI_HEADER_REPLACE, &ctr TSRMLS_CC);
+				}
+				/* the parser would return 1 (failure), we can bail out nicely */
+				if (type != E_PARSE) {
+					/* restore memory limit */
+					zend_set_memory_limit(PG(memory_limit));
+					zend_objects_store_mark_destructed(&EG(objects_store) TSRMLS_CC);
+					zend_bailout();
+					return;
+				}
+			}
+			break;
+	}
+
+#endif
+
 	if (PG(track_errors) && EG(active_symbol_table)) {
 		zval *tmp;
 
@@ -797,23 +864,11 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 	if (edata && edata->op_array) {
 		/* Normal function calls */
 		tmp->filename  = xdstrdup(edata->op_array->filename);
-	} else if (edata &&
-		edata->prev_execute_data &&
-		XDEBUG_LLIST_TAIL(XG(stack))
+	} else if (edata && edata->prev_execute_data && XDEBUG_LLIST_TAIL(XG(stack))
 	) {
-		/* Ugly hack for call_user_*() type function calls */
-		zend_function *tmpf = edata->prev_execute_data->function_state.function;
-		if (tmpf && (tmpf->common.type != 3) && tmpf->common.function_name) {
-			if (
-				(strcmp(tmpf->common.function_name, "call_user_func") == 0) ||
-				(strcmp(tmpf->common.function_name, "call_user_func_array") == 0) ||
-				(strcmp(tmpf->common.function_name, "call_user_func_method") == 0) ||
-				(strcmp(tmpf->common.function_name, "call_user_func_method_array") == 0)
-			) {
-				tmp->filename = xdstrdup(((function_stack_entry*) XDEBUG_LLIST_VALP(XDEBUG_LLIST_TAIL(XG(stack))))->filename);
-			}
-		}
+		tmp->filename = xdstrdup(((function_stack_entry*) XDEBUG_LLIST_VALP(XDEBUG_LLIST_TAIL(XG(stack))))->filename);
 	}
+
 	if (!tmp->filename) {
 		/* Includes/main script etc */
 		tmp->filename  = (op_array && op_array->filename) ? xdstrdup(op_array->filename): NULL;
@@ -848,7 +903,7 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 		}
 
 		if (tmp->function.type == XFUNC_EVAL) {
-			tmp->include_filename = xdebug_sprintf("'%s'", XG(last_eval_statement));
+			tmp->include_filename = xdebug_sprintf("%s", XG(last_eval_statement));
 		} else if (XG(collect_includes)) {
 			tmp->include_filename = xdstrdup(zend_get_executed_filename(TSRMLS_C));
 		}
@@ -858,6 +913,11 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 			if (cur_opcode) {
 				tmp->lineno = cur_opcode->lineno;
 			}
+		} else if (edata->prev_execute_data && edata->prev_execute_data->opline) {
+			cur_opcode = edata->prev_execute_data->opline;
+			if (cur_opcode) {
+				tmp->lineno = cur_opcode->lineno;
+			} 
 		}
 
 		if (XG(remote_enabled) || XG(collect_params) || XG(collect_vars)) {
